;********************************************************************
;
; Author        : ADI - Apps            www.analog.com/MicroConverter
;
; Date          : 15 October 1999
;
; File          : 812cal.asm    ******** PRE-RELEASE VERSION ********
;                                        ===================
; Hardware      : ADuC812
;
; Description   : offset and gain calibration routines for use with
;                 the ADuC812.  total size of assembled code is 330
;                 bytes.  routines for use external to this file are:
;
;                 CALZEROSCALE - calibrates ADC for offset
;                 CALFULLSCALE - calibrates ADC for gain
;
; NOTE          : this is a PRE-RELEASE version.  when released, this
;                 code will be available on the MicroConverter web
;                 site and will be complemented by a technical note
;                 describing the calibration routines in detail.
;
;********************************************************************

;____________________________________________________________________
                                                       ; CALZEROSCALE

CALZEROSCALE:   ; calibrates the ADC for zero scale on channel passed
                ; in via ACC.  the selected channel must be externaly
                ; held at a fixed voltage (within 5% of zero volts)
                ; throughout the the execution of this routine.
                ; routine does not modify any memory locations except
                ; for C.  C is returned to indicate success (C=0) or
                ; failure (C=1) of the calibration.  routine also
                ; uses the stack to 8 bytes deep.

; STORE REGISTER VALUES TO BE RECALLED LATER....

        PUSH    IE
        CLR     EA              ; disable all interrupts
        PUSH    ACC
        PUSH    B
        PUSH    0               ; address 0 is used as a count byte
        PUSH    ADCCON1
        PUSH    ADCCON2

; CONFIGURE ADC FOR CALIBRATION....
        ; note:  you can skip the below 8 lines (thru CALL DELAY65ms)
        ;        if the ADC is already configured in normal mode
        ;        with no convert-start events enabled.
        ;          i.e.  ADCCON1 = 01XXXX00 binary
        ;                ADCCON2 = X000XXXX binary
        ;          (where "X" indicats a "don't care" bit)
        ;        however, the delay will still be necessary if the
        ;        ADC was turned on less than 65ms prior to calling
        ;        this routine.

        CLR     CCONV           ; ensure ADC cont.conv. not active
        ANL     A,#7            ; ensure ACC is between 0 & 7..
        MOV     ADCCON2,A       ; channel number is passed in via ACC
        MOV     A,ADCCON1
        ANL     A,#01111100b    ; ensure ADC in normal mode and..
        ORL     A,#01000000b    ; ..timer/external CONVST disabled
        MOV     ADCCON1,A
        CALL    DELAY65ms       ; delay 65ms for Vref to turn on

        MOV     A,ADCCON3       ; if ADC is currently converting..
        JB      ACC.7,$-2       ; ..then wait for it to complete

        CLR     A
        MOV     ADCOFSH,A       ; start with zeros in cal registers
        MOV     ADCOFSL,A

; CALLIBRATE UPPER BYTE OF ADC OFFSET REGISTER (ADCOFSH)....

                                ; a single bit of ACC is set for use
        MOV     A,#20h          ; as a test bit.  start w/ 00100000b
                                ; to test ADCOFSH.5 (MSB) first.

CL0001: ORL     ADCOFSH,A       ; set test bit and check results..
        CALL    CONV10ZS        ; CONV10ZS returns C=(avg.conv>=0.5)
        JC      CL0002          ; if C, then leave test bit set
        CPL     A               ;
        ANL     ADCOFSH,A       ; else, clear test bit
        CPL     A               ;
CL0002: CLR     C               ;
        RRC     A               ; rotate A to point to next test bit
        JNC     CL0001          ; done when test bit rolls into C

; CALLIBRATE LOWER BYTE OF ADC OFFSET REGISTER (ADCOFSL)....

                                ; a single bit of ACC is set for use
        MOV     A,#80h          ; as a test bit.  start w/ 10000000b
                                ; to test ADCOFSL.7 (MSB) first.

CL0003: ORL     ADCOFSL,A       ; set test bit and check results..
        CALL    CONV10ZS        ; CONV10ZS returns C=(avg.conv>=0.5)
        JC      CL0004          ; if C, then leave test bit set
        CPL     A               ;
        ANL     ADCOFSL,A       ; else, clear test bit
        CPL     A               ;
CL0004: CLR     C               ;
        RRC     A               ; rotate A to point to next test bit
        JNC     CL0003          ; done when test bit rolls into C


; ENSURE CAL COEFFICIENTS ARE WITHIN RANGE (i.e. not saturated)....

        MOV     A,ADCOFSH       ; return "fail" if ADCOFS < 00FFh
        JNZ     CL0005
        SETB    C               ; set C if ADCOFSH
        JMP     CL0006

CL0005: MOV     A,ADCOFSH       ; return "fail" if ADCOFS > 3EFFh
        CJNE    A,#03Eh,$+3     ; C becomes: C = (ADCOFSH<=3Eh)
        CPL     C               ; cpl C so.. C = (ADCOFSH>3Eh)

        ; C=1 means "fail", i.e. cal coefficients are saturated
        ; C=0 means "pass", i.e. cal coefficients are within range

; RESTORE STORED REGISTER VALUES AND RETURN....

CL0006: POP     ADCCON2
        POP     ADCCON1
        POP     0
        POP     B
        POP     ACC
        POP     IE
        RET

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

CONV10ZS:       ; performs 10 ADC conversions to determine if current
                ; test bit should remain set, or should be cleared to
                ; zero.  returns C as an answer.  if the average of
                ; all 10 conversions is less than 0.5, then C=0 is
                ; returned indicating that the test bit should be
                ; cleared to zero.  otherwise, C=1 is returned
                ; indicating that the test bit should remain set.

        PUSH    ACC

        CLR     ADCI
        SETB    SCONV           ; dummy conversion, ignore 1st result
        JNB     ADCI,$
        MOV     0,#10           ; number of conversions to take
        CLR     A
        MOV     B,A             ; B stores running sum
CL0007: CLR     ADCI
        SETB    SCONV           ; convert
        JNB     ADCI,$
        MOV     A,ADCDATAH
        ANL     A,#0Fh          ; remove channel ID
        JNZ     CL0008          ; if hi-byte>0, then return C=0
        MOV     A,B
        ADD     A,ADCDATAL      ; add result to running summ
        MOV     B,A
        JC      CL0008          ; if sum ovrflw, then return C=0
        DJNZ    0,CL0007        ; addr 0 contains count byte

        MOV     A,B
        CJNE    A,#5,$+3        ; if sum<5, C is set, else cleared
        CPL     C               ; cpl C, so C = (sum>=5)
        POP     ACC             ; i.e., return: C = (average>=0.5)
        RET

CL0008: SETB    C               ; if sum overflowed during routine..
        POP     ACC             ; ..then set C and return
        RET

;____________________________________________________________________
                                                       ; CALFULLSCALE

CALFULLSCALE:   ; calibrates the ADC for full scale on channel passed
                ; in via ACC.  the selected channel must be externaly
                ; held at a fixed voltage (within 2.5% of Vref)
                ; throughout the the execution of this routine.
                ; routine does not modify any memory locations except
                ; for C.  C is returned to indicate success (C=0) or
                ; failure (C=1) of the calibration.  routine also
                ; uses the stack to 8 bytes deep.

; STORE REGISTER VALUES TO BE RECALLED LATER....

        PUSH    IE
        CLR     EA              ; disable all interrupts
        PUSH    ACC
        PUSH    B
        PUSH    0               ; address 0 is used as a count byte
        PUSH    ADCCON1
        PUSH    ADCCON2

; CONFIGURE ADC FOR CALIBRATION....
        ; note:  you can skip the below 8 lines (thru CALL DELAY65ms)
        ;        if the ADC is already configured in normal mode
        ;        with no convert-start events enabled.
        ;          i.e.  ADCCON1 = 01XXXX00 binary
        ;                ADCCON2 = X000XXXX binary
        ;          (where "X" indicats a "don't care" bit)
        ;        however, the delay will still be necessary if the
        ;        ADC was turned on less than 65ms prior to calling
        ;        this routine.

        CLR     CCONV           ; ensure ADC cont.conv. not active
        ANL     A,#7            ; ensure ACC is between 0 & 7..
        MOV     ADCCON2,A       ; channel number is passed in via ACC
        MOV     A,ADCCON1
        ANL     A,#01111100b    ; ensure ADC in normal mode and..
        ORL     A,#01000000b    ; ..timer/external CONVST disabled
        MOV     ADCCON1,A
        CALL    DELAY65ms       ; delay 65ms for Vref to turn on

        MOV     A,ADCCON3       ; if ADC is currently converting..
        JB      ACC.7,$-2       ; ..then wait for it to complete

        CLR     A
        MOV     ADCGAINH,A      ; start with zeros in cal registers
        MOV     ADCGAINL,A

; CALLIBRATE UPPER BYTE OF ADC GAIN REGISTER (ADCGAINH)....

                                ; a single bit of ACC is set for use
        MOV     A,#20h          ; as a test bit.  start w/ 00100000b
                                ; to test ADCGAINH.5 (MSB) first.

CL1001: ORL     ADCGAINH,A      ; set test bit and check results..
        CALL    CONV10FS        ; CONV10FS rtrns C=(avg.conv<4095.5)
        JC      CL1002          ; if C, then leave test bit set
        CPL     A               ;
        ANL     ADCGAINH,A      ; else, clear test bit
        CPL     A               ;
CL1002: CLR     C               ;
        RRC     A               ; rotate A to point to next test bit
        JNC     CL1001          ; done when test bit rolls into C

; CALLIBRATE LOWER BYTE OF ADC GAIN REGISTER (ADCGAINL)....

                                ; a single bit of ACC is set for use
        MOV     A,#80h          ; as a test bit.  start w/ 10000000b
                                ; to test ADCGAINL.7 (MSB) first.

CL1003: ORL     ADCGAINL,A      ; set test bit and check results..
        CALL    CONV10FS        ; CONV10FS rtrns C=(avg.conv<4095.5)
        JC      CL1004          ; if C, then leave test bit set
        CPL     A               ;
        ANL     ADCGAINL,A      ; else, clear test bit
        CPL     A               ;
CL1004: CLR     C               ;
        RRC     A               ; rotate A to point to next test bit
        JNC     CL1003          ; done when test bit rolls into C


; ENSURE CAL COEFFICIENTS ARE WITHIN RANGE (i.e. not saturated)....

        MOV     A,ADCGAINH      ; return "fail" if ADCGAIN < 00FFh
        JNZ     CL1005
        SETB    C               ; set C if ADCGAINH is zero
        JMP     CL1006

CL1005: MOV     A,ADCGAINH      ; return "fail" if ADCGAIN > 3EFFh
        CJNE    A,#03Eh,$+3     ; C becomes: C = (ADCGAINH<=3Eh)
        CPL     C               ; cpl C so.. C = (ADCGAINH>3Eh)

        ; C=1 means "fail", i.e. cal coefficients are saturated
        ; C=0 means "pass", i.e. cal coefficients are within range

; RESTORE STORED REGISTER VALUES AND RETURN....

CL1006: POP     ADCCON2
        POP     ADCCON1
        POP     0
        POP     B
        POP     ACC
        POP     IE
        RET

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

CONV10FS:       ; performs 10 ADC conversions to determine if current
                ; test bit should remain set, or should be cleared to
                ; zero.  returns C as an answer.  if the average of
                ; all 10 conversions is less than 4094.5, then C=1 is
                ; returned indicating that the test bit should remain
                ; set.  otherwise, C=0 is returned indicating that
                ; the test bit needs to be cleared.
                ; the "sum" referred to below is actually a sum of
                ; the values (4095-result).  therefore, final "sum"
                ; will be between 0 & 5 when the average of the 10
                ; conversion results is between 4094.5 & 4095.

        PUSH    ACC

        CLR     ADCI
        SETB    SCONV           ; dummy conversion, ignore 1st result
        JNB     ADCI,$
        MOV     0,#10           ; number of conversions to take
        CLR     A
        MOV     B,A             ; B stores running sum
CL1007: CLR     ADCI
        SETB    SCONV           ; convert
        JNB     ADCI,$
        MOV     A,ADCDATAH      ; ADCDATAH should be _F hex
        INC     A               ; ACC should now contain _0 hex
        ANL     A,#0Fh          ; remove channel ID
        JNZ     CL1009          ; if hi-byte<0F, then return C=1
        MOV     A,ADCDATAL
        CPL     A               ; CPL of ADCDATAL = (FF-ADCDATAL)
        ADD     A,B             ; add result to running sum
        MOV     B,A
        JC      CL1009          ; if sum ovrflw, then return C=1
        DJNZ    0,CL1007        ; addr 0 contains count byte

        MOV     A,B
        CJNE    A,#5,CL1008     ; if sum<5, C is set, else cleared
CL1008: CPL     C               ; cpl C, so....
        POP     ACC             ; return: C = (sum>5)
        RET                     ; i.e.:   C = (average<4094.5)

CL1009: SETB    C               ; if sum overflowed during routine..
        POP     ACC             ; ..then set C and return
        RET

;____________________________________________________________________
                                                          ; DELAY65ms

DELAY65ms:                      ; delay for 65ms

        PUSH    ACC
        PUSH    B

        MOV     B,#130          ; 130 * 500us = 65ms
DLY001: MOV     A,#229          ; 229 * 2.17us = 500us
        DJNZ    ACC,$           ; sit here for 500us
        DJNZ    B,DLY001        ; repeat 130 times (65ms delay)

        POP     B
        POP     ACC
        RET

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

